/*
   https://cscircles.cemc.uwaterloo.ca/java_visualize/
   http://pythontutor.com/java.html#mode=edit
*/

public class VisualizeMatrixOperations_ver1
{
   public static void main(String[] args)
   {
      Mat2 m1 = new Mat2(2, 0, 0, 2);
      Mat2 m2 = new Mat2(3, 0, 0, 3);
      Mat2 m3 = new Mat2(4, 0, 0, 4);

      // The following two lines of code act very differently!
      Mat2 m4 = m3.times(m2).times(m1); // m4 = (m3 * m2) * m1
      Mat2 m5 = m3.mult(m2).mult(m1);   // Bad idea! (Why?)

      // The following two lines of code also act very differently!
      Mat2 m6 = m3.times(m2.times(m1)); // m6 = m3 * (m2 * m1)
      Mat2 m7 = m3.mult(m2.mult(m1));   // Even worse idea! (Why?)

      // Why is the next line a reasonable idea?
      Mat2 m8 = new Mat2(1,0,0,1).mult(m3).mult(m2).mult(m1);

      System.out.println( m1 );
      System.out.println( m2 );
      System.out.println( m3 );
      System.out.println( m4 );
      System.out.println( m5 );
      System.out.println( m6 );
      System.out.println( m7 );
      System.out.println( m8 );
   }
}


class Mat2
{
   private double m00, m01, m10, m11;

   public Mat2(double m00, double m01, double m10, double m11)
   {
      //System.out.println(this.hashCode());
      this.m00 = m00;
      this.m01 = m01;
      this.m10 = m10;
      this.m11 = m11;
   }

   public Mat2 times(final Mat2 that) // matrix times matrix returns matrix
   {
      //System.out.println(this.hashCode() + "  " + that.hashCode());
      final double a00 = this.m00 * that.m00 + this.m01 * that.m10;
      final double a01 = this.m00 * that.m01 + this.m01 * that.m11;
      final double a10 = this.m10 * that.m00 + this.m11 * that.m10;
      final double a11 = this.m10 * that.m10 + this.m11 * that.m11;
      return new Mat2(a00, a01, a10, a11); // return a new matrix
   }

   public Mat2 mult(final Mat2 that) // matrix times matrix mutates matrix
   {
      //System.out.println(this.hashCode() + "  " + that.hashCode());
      final double a00 = this.m00 * that.m00 + this.m01 * that.m10;
      final double a01 = this.m00 * that.m01 + this.m01 * that.m11;
      final double a10 = this.m10 * that.m00 + this.m11 * that.m10;
      final double a11 = this.m10 * that.m10 + this.m11 * that.m11;
      this.m00 = a00; // Mutate this Mat2 object.
      this.m01 = a01;
      this.m10 = a10;
      this.m11 = a11;
      return this; // For method chaining.
   }

   public String toString(){return "["+m00+",\t"+m01+"]\n["+m10+",\t"+m11+"]";}
}

/*
Notice the following important failing of the Java language.

The times() method has this signature,

   public Mat2 times(Mat2 m)

and it is applied like this,

   m3.times(m2).times(m1)

The mult() method has essentially the same signature,

   public Mat2 mult(Mat2 m)

and it is applied in essentially the same way,

   m3.mult(m2).mult(m1)

So there is nothing in the Java syntax, nor in the Java
type system, that helps you realize that mult() mutates
the object it is called on, while times() returns a new
object. You can only tell the difference between the
two methods by reading the details of their source code.

Another way to put this is that a method's interface,
its signature,

   public Mat2 times(Mat2 m)
   public Mat2  mult(Mat2 m)

tells you very little about what the method does. The
interface of a method does not work as a specification.
In some modern functional programming languages (such as
Haskell) interfaces are much better at being specifications.
*/
