/*
   https://cscircles.cemc.uwaterloo.ca/java_visualize/
   http://pythontutor.com/java.html#mode=edit
*/

public class VisualizeMatrixOperations_ver3
{
   public static void main(String[] args)
   {
      Mat2 m1 = new Mat2(2, 0, 0, 2);
      Mat2 m2 = new Mat2(3, 0, 0, 3);
      Vec2 v1 = new Vec2(4, 4);

      //   v2 = (m1 * m2) * v1
      Vec2 v2 = m1.times(m2).times(v1);
      Vec2 v3 = m1.mult(m2).times(v1);
      Vec2 v4 = v1.mult(m1.times(m2));
      Vec2 v5 = v1.mult(m1.mult(m2));

      //   v2 = m1 * (m2 * v1).
      Vec2 v6 = m1.times(m2.times(v1));
      Vec2 v7 = v1.mult(m2).mult(m1);

      System.out.println( m1 );
      System.out.println( m2 );
      System.out.println( v1 );
      System.out.println( v2 );
   }
}

class Vec2
{
   public double x, y;

   public Vec2(double x, double y)
   {
      this.x = x;
      this.y = y;
   }

   public Vec2 plus(final Vec2 v) // vector plus vector returns vector
   {
      return new Vec2( x + v.x, y + v.y );
   }

   public Vec2 mult(final Mat2 that) // matrix times vector mutates vector
   {
      final double x2 = (that.v1.x * this.x) + (that.v2.x * this.y);
      final double y2 = (that.v1.y * this.x) + (that.v2.y * this.y);
      this.x = x2;   // Mutate this Vec2 object.
      this.y = y2;
      return this;   // For method chaining.
   }

   public String toString(){return "["+x+",\t"+y+"]";}
}


class Mat2
{
   public Vec2 v1, v2; // These are column vectors.

   public Mat2(double m00, double m01, double m10, double m11)
   {
      this.v1 = new Vec2(m00, m10); // 1st column
      this.v2 = new Vec2(m01, m11); // 2nd column
   }

   public Mat2 times(final Mat2 that) // matrix times matrix returns matrix
   {
      final Vec2 vec1 = this.times(that.v1);
      final Vec2 vec2 = this.times(that.v2);
      return new Mat2(vec1.x, vec2.x, vec1.y, vec2.y);
   }

    public Vec2 times(final Vec2 v) // matrix times vector returns vector
    {
       final double x = (v1.x * v.x) + (v2.x * v.y);
       final double y = (v1.y * v.x) + (v2.y * v.y);
       return new Vec2(x, y);
   }

   public Mat2 mult(final Mat2 that) // matrix times matrix mutates matrix
   {
      final Vec2 vec1 = this.times(that.v1);
      final Vec2 vec2 = this.times(that.v2);
      this.v1 = vec1; // Mutate this Mat2 object.
      this.v2 = vec2;
      return this;    // For method chaining.
   }

   public String toString(){return v1+"\n"+v2;}
}

/*
Notice the following important failing of the Java language.

The times() method has this signature,

   public Mat2 times(Mat2 m)

and it is applied like this,

   m3.times(m2).times(m1)

The mult() method has essentially the same signature,

   public Mat2 mult(Mat2 m)

and it is applied in essentially the same way,

   m3.mult(m2).mult(m1)

So there is nothing in the Java syntax, nor in the Java
type system, that helps you realize that mult() mutates
the object it is called on, while times() returns a new
object. You can only tell the difference between the
two methods by reading the details of their source code.

Another way to put this is that a method's interface,
its signature,

   public Mat2 times(Mat2 m)
   public Mat2  mult(Mat2 m)

tells you very little about what the method does. The
interface of a method does not work as a specification.
In some modern functional programming languages (such as
Haskell) interfaces are much better at being specifications.
*/
