/*
   https://cscircles.cemc.uwaterloo.ca/java_visualize/
   http://pythontutor.com/java.html#mode=edit
*/


/**
   This version implements 1-by-1 matrices.
   A 1-by-1 matrix is just one number, but
   that number is treated as a matrix.

   This version makes it easier to see in
   the Java Visualizer what objects are
   being created and mutated.
*/
public class VisualizeMatrixOperations_ver4
{
   public static void main(String[] args)
   {
      Mat1 m1 = new Mat1(2);  // These are 1-by-1 matrices.
      Mat1 m2 = new Mat1(3);
      Mat1 m3 = new Mat1(4);

      // The following two lines of code act very differently!
      Mat1 m4 = m3.times(m2).times(m1); // m4 = (m3 * m2) * m1
      Mat1 m5 = m3.mult(m2).mult(m1);   // Bad idea! (Why?)

      // The following two lines of code also act very differently!
      Mat1 m6 = m3.times(m2.times(m1)); // m6 = m3 * (m2 * m1)
      Mat1 m7 = m3.mult(m2.mult(m1));   // Even worse idea! (Why?)

      // Why is the next line a reasonable idea?
      Mat1 m8 = new Mat1(1).mult(m3).mult(m2).mult(m1);

      System.out.println( m1 );
      System.out.println( m2 );
      System.out.println( m3 );
      System.out.println( m4 );
      System.out.println( m5 );
      System.out.println( m6 );
      System.out.println( m7 );
      System.out.println( m8 );
   }
}


class Mat1
{
   private double m00;

   public Mat1(double m00)
   {
      //System.out.println(this.hashCode());
      this.m00 = m00;
   }

   public Mat1 times(final Mat1 that) // matrix times matrix returns matrix
   {
      //System.out.println(this.hashCode() + "  " + that.hashCode());
      final double a00 = this.m00 * that.m00;
      return new Mat1(a00); // return a new matrix
   }

   public Mat1 mult(final Mat1 that) // matrix times matrix mutates matrix
   {
      //System.out.println(this.hashCode() + "  " + that.hashCode());
      final double a00 = this.m00 * that.m00;
      this.m00 = a00; // Mutate this Mat2 object.
      return this; // For method chaining.
   }

   public String toString(){return "[ "+m00+" ]";}
}
